@@ -9,9 +9,15 @@
 #include <vector>
 #include <string.h>
 #include <cassert>
+#include <netdb.h>
+#include <sys/types.h>
+#include <netinet/in.h>
+#include <sys/socket.h>
+#include <sys/wait.h>
 #include "Engine.h"
 #include "MainWindow.h"
 #include "Sound.h"
+#include "ec_throw.h"
 
 /** 边界的宽度*/
 /**  width of border */
@@ -103,13 +109,17 @@ Board::Board(MainWindow& win) :
 	m_status(FREE_STATUS),
 	ui_pixmap(NULL),
 	p_pgnfile(NULL),
+	fd_recv_skt(-1),
+	fd_send_skt(-1),
 	selected_chessman(-1)
 	,postion_str("position fen ")
 	,parent(win)
-	,red_time(600)
-	,black_time(600)
+	,red_time(2400)
+	,black_time(2400)
+	,play_time(40)
+	,step_time(240)
 	,count_time(0)
-	,limit_count_time(60)
+	,limit_count_time(240)
 	,chessman_width(29)
 	,is_small_board(true)
 	,is_rev_board(false)
@@ -136,6 +146,7 @@ Board::Board(MainWindow& win) :
 	p_pgnfile=new Pgnfile(m_engine);
 	m_engine.init_snapshot(start_fen);
 	m_robot.set_out_slot(sigc::mem_fun(*this, &Board::robot_log));
+	//m_network.set_out_slot(sigc::mem_fun(*this, &Board::network_log));
 	this->set_events(Gdk::BUTTON_PRESS_MASK|Gdk::EXPOSURE_MASK);
 
 	this->show_all();
@@ -150,6 +161,7 @@ Board::~Board()
 		timer.disconnect();
 	m_robot.send_ctrl_command("quit\n");
 	m_robot.stop();
+	//m_network.stop();
 }
 
 void Board::load_images()
@@ -297,7 +309,7 @@ bool Board::on_button_press_event(GdkEventButton* ev)
 {
 	/** 对战状态下，电脑走棋时就不响应鼠标事件了*/
 	/** if fight with AI, the compute doesn't handle the events */
-	if(is_fight_to_robot()){
+	if(is_fight_to_robot()||is_network_game()){
 		if(!is_human_player())
 			return true;
 
@@ -322,9 +334,9 @@ bool Board::on_button_press_event(GdkEventButton* ev)
 					printf("still no select chessman\n");
 					return true;
 				}
-				/** 对战状态中，选了黑色棋子无效*/
-				/** choose the black chessman is useless on war */
-				if(is_fight_to_robot()){ 
+				/** 对战状态中，选了对方棋子无效*/
+				/** choose the enemy chessman is useless on war */
+				if(is_fight_to_robot()||is_network_game()){ 
 					if((m_human_black && (selected_chessman <32))||((!m_human_black)&&(selected_chessman>31))){
 						printf("choose black %d\n",selected_chessman);
 						selected_chessman =-1;
@@ -596,7 +608,7 @@ void Board::draw_board()
 	}
 }
 
-void Board::start_move()
+void Board::first_move()
 {
 	m_step = 0;
 	m_engine.get_snapshot(m_step);
@@ -610,10 +622,11 @@ void Board::start_move()
 	else
 		parent.set_comment(" ");
 
+	CSound::play(SND_MOVE);
 	redraw();
 }
 
-void Board::end_move()
+void Board::last_move()
 {
 	m_step = m_engine.how_step();
 	m_engine.get_snapshot(m_step);
@@ -627,6 +640,7 @@ void Board::end_move()
 	else
 		parent.set_comment(" ");
 
+	CSound::play(SND_MOVE);
 	redraw();
 }
 void Board::next_move()
@@ -730,6 +744,7 @@ int Board::try_move(int mv)
 		printf("sleceted = %d finish move and redraw now\n",selected_chessman);
 		selected_chessman=-1;
 
+		std::string iccs_str=m_engine.move_to_iccs_str(mv);
 		/** 对战时的处理*/
 		if(is_fight_to_robot()){
 			if(eat){
@@ -737,7 +752,6 @@ int Board::try_move(int mv)
 				moves_lines =postion_str+ m_engine.get_last_fen_from_snapshot()+std::string(" -- 0 1 ");
 			}
 			else{
-				std::string iccs_str=m_engine.move_to_iccs_str(mv);
 				size_t pos = moves_lines.find("moves");
 				if(pos == std::string::npos){
 					moves_lines=moves_lines + " -- 0 1  moves "+iccs_str;
@@ -764,6 +778,20 @@ int Board::try_move(int mv)
 			parent.change_play(is_human_player());
 			count_time=0;
 		}
+		else if(is_network_game()){
+
+			if(!is_human_player()){
+			/** 我走的棋，则将iccs_str走法传给网络*/
+				printf("my go\n");
+				std::string mv_str = "moves:"+iccs_str;
+				send_to_socket(mv_str);
+
+
+			}
+			parent.change_play(is_human_player());
+			count_time=0;
+
+		}
 
 		/**被将死了*/
 		/** is it  mate */
@@ -797,6 +825,9 @@ void Board::draw_move()
 			m_robot.send_ctrl_command("ponderhit draw\n");
 
 	}
+	else if(is_network_game()){
+		parent.on_end_game(ROBOT_DRAW);
+	}
 
 }
 
@@ -807,7 +838,6 @@ void Board::rue_move()
 		return;
 	DLOG(" how_step %d\n",m_engine.how_step());
 	int mv = m_engine.get_last_move_from_snapshot();
-	//DLOG(" undo move %d\n");
 	m_engine.undo_move(mv);
 	parent.del_step_last_line();
 
@@ -818,7 +848,6 @@ void Board::rue_move()
 		moves_lines =postion_str+ m_engine.get_last_fen_from_snapshot()+std::string(" -- 0 1 ");
 		m_robot.send_ctrl_command(moves_lines.c_str());
 		m_robot.send_ctrl_command("\n");
-		//user_player = 1-user_player;
 	}
 }
 
@@ -855,7 +884,7 @@ void Board::on_drog_data_received(const Glib::RefPtr<Gdk::DragContext>& context,
 	}
 }
 
-void Board::free_game()
+void Board::free_game(bool redraw_)
 {
 	if(timer.connected())
 		timer.disconnect();
@@ -863,11 +892,13 @@ void Board::free_game()
 	m_robot.send_ctrl_command("quit\n");
 	m_robot.stop();
 	m_status = FREE_STATUS;
-	red_time=600;
-	black_time=600;
+	is_rev_board=false;
+	m_human_black=false;
 
-	m_engine.init_snapshot(start_fen);
-	redraw();
+	if(redraw_){
+		m_engine.init_snapshot(start_fen);
+		redraw();
+	}
 }
 void Board::rev_game()
 {
@@ -882,7 +913,7 @@ void Board::start_robot()
 
 	m_robot.start();
 	m_robot.send_ctrl_command("ucci\n");
-	new_game();
+	new_game(m_status);
 }
 
 void Board::set_level_config(int _depth,int _idle,int _style,int _knowledge,int _pruning,int _randomness,bool _usebook)
@@ -892,9 +923,17 @@ void Board::set_level_config(int _depth,int _idle,int _style,int _knowledge,int
 
 }
 
+void Board::set_time(int _step_time, int _play_time)
+{
+	step_time = _step_time;
+	play_time = _play_time;
+}
+
 void Board::set_war_time(int _step_time,int _play_time)
 {
 	limit_count_time = _step_time;
+	red_time = _play_time*60;
+	black_time = _play_time* 60;
 
 }
 
@@ -913,13 +952,24 @@ void Board::set_level()
 		m_robot.send_ctrl_command("setoption usebook true\n");
 }
 
-void Board::new_game()
+void Board::start_network()
 {
-	m_status = FIGHT_STATUS;
+	set_war_time(300,30);
+	new_game(NETWORK_STATUS);
+
+}
+
+void Board::new_game(BOARD_STATUS _status)
+{
+	m_status = _status;
 
 	m_engine.init_snapshot(start_fen);
-	m_robot.send_ctrl_command("setoption newgame\n");
-	set_level();
+
+	if(m_status == FIGHT_STATUS){
+		set_war_time(step_time,play_time);
+		m_robot.send_ctrl_command("setoption newgame\n");
+		set_level();
+	}
 
 
 	moves_lines.clear();
@@ -927,23 +977,29 @@ void Board::new_game()
 	redraw();
 
 	parent.textview_engine_log_clear();
-	//parent.change_play(m_engine.red_player());
 	parent.change_play(is_human_player());
 
 	timer=Glib::signal_timeout().connect(sigc::mem_fun(*this,&Board::go_time),1000);
 	/**如果是用户选择黑方，则电脑先走棋 -- if user choose black,the robot go moves first*/
 	if(m_human_black){
-		moves_lines =moves_lines +std::string(" -- 0 1 ");
-		m_robot.send_ctrl_command(moves_lines.c_str());
-		m_robot.send_ctrl_command("\n");
-		char str_cmd[256];
-		sprintf(str_cmd,"go depth %d \n",m_search_depth);
-		m_robot.send_ctrl_command(str_cmd);
+		if(m_status == FIGHT_STATUS){
+			moves_lines =moves_lines +std::string(" -- 0 1 ");
+			m_robot.send_ctrl_command(moves_lines.c_str());
+			m_robot.send_ctrl_command("\n");
+			char str_cmd[256];
+			sprintf(str_cmd,"go depth %d \n",m_search_depth);
+			m_robot.send_ctrl_command(str_cmd);
+		}else if(m_status == NETWORK_STATUS){
+
+		}
 
 	}
 
+	parent.set_red_war_time(to_time_ustring(red_time),to_time_ustring(0));
+	parent.set_black_war_time(to_time_ustring(black_time),to_time_ustring(0));
 }
 
+
 bool Board::robot_log(const Glib::IOCondition& condition)
 {
 	/*for testing,delete me*/
@@ -1033,6 +1089,8 @@ bool Board::go_time()
 				timer.disconnect();
 			parent.on_end_game(HUMAN_OVER_TIME);
 			count_time =0;
+			if(is_network_game())
+				send_to_socket("timeout");
 
 		}
 	}
@@ -1051,6 +1109,8 @@ bool Board::go_time()
 				timer.disconnect();
 			parent.on_end_game(ROBOT_OVER_TIME);
 			count_time =0;
+			if(is_network_game())
+				send_to_socket("timeout");
 		}
 	}
 
@@ -1117,3 +1177,200 @@ void Board::reckon_time_sound(int time_)
 	}
 
 }
+
+void Board::watch_socket(int fd)
+{
+	fd_recv_skt=fd;
+	Glib::signal_io().connect(sigc::mem_fun(*this,&Board::on_network_io),
+			fd_recv_skt, Glib::IO_IN);
+
+}
+bool Board::on_network_io(const Glib::IOCondition& )
+{
+
+	int fd_cli = -1;
+       EC_THROW(-1 == (fd_cli = accept(fd_recv_skt, NULL, 0)));
+       char buf[1024];
+       size_t len = read(fd_cli, &buf[0], 1023);
+       buf[len]=0;
+       if (len > 0) {
+
+		std::string str_buf(buf);
+		size_t pos;
+		//size_t pos_=str_buf.find("network-game-red");
+
+		if((pos = str_buf.find("moves:")) != std::string::npos){
+			std::string t_mv=str_buf.substr(pos+6,4);
+			std::cout<<"get robot mv = "<<t_mv<<std::endl;
+			int mv = m_engine.iccs_str_to_move(t_mv);
+			try_move(mv);
+
+		}else if((pos = str_buf.find("network-game-red,"))!= std::string::npos){
+			//start network game with red player
+			std::string enemy_name,my_name;
+			size_t pos_s,pos_e,pos_m;
+			pos_s= str_buf.find("enemy_name:");
+			pos_e= str_buf.find(",my_name:");
+			pos_m= str_buf.find_first_of("@");
+			enemy_name = str_buf.substr(pos_s+11,pos_m-pos_s-11);
+			pos_m = str_buf.find_last_of("@");
+			my_name = str_buf.substr(pos_e+9,pos_m-pos_e-9);
+			
+			//parent.on_network_game(my_name,enemy_name,true);
+			parent.on_network_game(enemy_name,my_name,true);
+		}else if((pos = str_buf.find("network-game-black,")) != std::string::npos){
+			//start network game with black player
+			std::string enemy_name,my_name;
+			size_t pos_s,pos_e,pos_m;
+			pos_s= str_buf.find("enemy_name:");
+			pos_e= str_buf.find(",my_name:");
+			pos_m= str_buf.find_first_of("@");
+			enemy_name = str_buf.substr(pos_s+11,pos_m-pos_s-11);
+			pos_m = str_buf.find_last_of("@");
+			my_name = str_buf.substr(pos_e+9,pos_m-pos_e-9);
+			parent.on_network_game(my_name,enemy_name,false);
+		}else if(str_buf.find("network-game-win") != std::string::npos){
+			if(timer.connected())
+				timer.disconnect();
+			parent.on_end_game(ROBOT_LOSE);
+			return true;
+		}else if(str_buf.find("resign") != std::string::npos){
+			if(timer.connected())
+				timer.disconnect();
+			parent.on_end_game(ROBOT_WIN);
+			return true;
+		}else if(str_buf.find("network-game-rue") != std::string::npos){
+			rue_move();
+			rue_move();
+		}else if(str_buf.find("network-game-norue") != std::string::npos){
+			parent.info_window("The against doesn't agree rue!");
+		}else if(str_buf.find("network-game-nodraw") != std::string::npos){
+			parent.info_window(("The against doesn't agree draw!"));
+		}else if(str_buf.find("network-game-draw") != std::string::npos){
+			if(timer.connected())
+				timer.disconnect();
+			parent.on_end_game(ROBOT_DRAW);
+		}else if(str_buf.find("enemy_name:") != std::string::npos){
+
+		}else if(str_buf.find("my_name:") !=std::string::npos){
+
+		}
+
+
+
+
+
+#if 0
+		if(pos_ != std::string::npos){
+			//start network game with red player
+			parent.on_network_game("lerosua","enemy",true);
+		}
+		pos_=str_buf.find("network-game-black");
+		if(pos_ != std::string::npos){
+			//start network game with black player
+			parent.on_network_game("lerosua","enemy",false);
+		}
+		pos_ = str_buf.find("network-game-win");
+		if(pos_ !=std::string::npos){
+			// i win the game.
+			if(timer.connected())
+				timer.disconnect();
+			parent.on_end_game(ROBOT_LOSE);
+			return true;
+		}
+		pos_ = str_buf.find("resign");
+		if(pos_ != std::string::npos){
+
+			if(timer.connected())
+				timer.disconnect();
+			parent.on_end_game(ROBOT_LOSE);
+			return true;
+		}
+		size_t pos=str_buf.find("moves:");
+		if(pos != std::string::npos){
+			std::string t_mv=str_buf.substr(pos+6,4);
+			std::cout<<"get robot mv = "<<t_mv<<std::endl;
+			int mv = m_engine.iccs_str_to_move(t_mv);
+			try_move(mv);
+		}
+
+		if(0 == strncmp("[{game:gmchess,",buf,15)){
+			printf("get cmd %s\n",buf);
+			string cmd=string(buf);
+			network_package* net_pac = parse_network_package(cmd);
+
+			if(net_pac->action=="ask"){
+				//对方请求的包
+				if(net_pac->status == "start"){
+					//对方请求游戏
+
+				}
+
+
+			}else if(net_pac->action == "reply"){
+				//对方回复的包
+
+
+			}else if(net_pac->action == "working"){
+				//对方走棋的包
+
+				int mv = m_engine.iccs_str_to_move(net_pac->moves);
+				try_move(mv);
+
+			}
+
+
+
+		}
+#endif
+	}
+	close(fd_cli);
+	return true;
+}
+
+
+int Board::init_send_socket()
+{
+	int sockfd;
+	char buf[1024];
+	struct sockaddr_in srvaddr;
+
+	EC_THROW(-1 == (sockfd=socket(AF_INET,SOCK_STREAM,0)));
+	bzero(&srvaddr,sizeof(srvaddr));
+	srvaddr.sin_family=AF_INET;
+	srvaddr.sin_port=htons(GMPORT+1);
+	srvaddr.sin_addr.s_addr=htonl(INADDR_ANY);
+
+	int on = 1;
+	EC_THROW( -1 == (setsockopt( sockfd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on) )));
+
+	return sockfd;
+}
+void Board::send_to_socket(const std::string& cmd_)
+{
+
+	int sockfd;
+	char buf[1024];
+	struct sockaddr_in srvaddr;
+
+	EC_THROW(-1 == (sockfd=socket(AF_INET,SOCK_STREAM,0)));
+	bzero(&srvaddr,sizeof(srvaddr));
+	srvaddr.sin_family=AF_INET;
+	srvaddr.sin_port=htons(GMPORT+1);
+	srvaddr.sin_addr.s_addr=htonl(INADDR_ANY);
+
+	int on = 1;
+	EC_THROW( -1 == (setsockopt( sockfd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on) )));
+
+	if( 0 == connect(sockfd,(struct sockaddr*)&srvaddr,sizeof(srvaddr))){
+		write(sockfd,cmd_.c_str(),cmd_.size());
+		close(sockfd);
+	}
+}
+void Board::close_send_socket()
+{
+	if(fd_send_skt>0){
+		close(fd_send_skt);
+		fd_send_skt=-1;
+	}
+}
