@@ -15,6 +15,7 @@
 #include "Engine.h"
 #include "Pgnfile.h"
 #include "robot.h"
+//#include "pidgin.h"
 
 class MainWindow;
 
@@ -33,6 +34,13 @@ class Board : public Gtk::DrawingArea
 		Board(MainWindow& win);
 		~Board();
 		const Board_info& get_board_info(){ return p_pgnfile->get_board_info() ;}
+		void watch_socket(int fd);
+		bool on_network_io(const Glib::IOCondition&);
+		int init_send_socket();
+		void close_send_socket();
+		void send_to_socket(const std::string& cmd_);
+
+
 	protected:
 		void on_map();
 		bool on_expose_event(GdkEventExpose* ev);
@@ -90,19 +98,23 @@ class Board : public Gtk::DrawingArea
 		void start_robot();
 		/** 开启新游戏，即在引擎已启动的情况下重新游戏*/
 		/** start a new game,with the presence engine*/
-		void new_game();
+		void new_game(BOARD_STATUS _status = FIGHT_STATUS);
 		/** 自由模式，即摆棋*/
 		/** free game mode*/
-		void free_game();
+		/** @param redraw 为真则重画棋盘，假则保留棋盘现状*/
+		void free_game(bool redraw=true);
+		/** 网络对战初始化*/
+		/** network play game initial*/
+		void start_network();
 		/** 读取AI的输出，并根据输出的着法走棋*/
 		/** read the output of AI,then go chess*/
 		bool robot_log(const Glib::IOCondition& condition);
 		/** 回到最初局面*/
 		/** return the first station*/
-		void start_move();
+		void first_move();
 		/** 去到最后的局面*/
 		/** go to the last station */
-		void end_move();
+		void last_move();
 		/** 读谱状态下走下一步棋*/
 		/**  go next on reading chessbook*/
 		void next_move();
@@ -145,8 +157,12 @@ class Board : public Gtk::DrawingArea
 		/** 标识是否与电脑AI对战*/
 		/** check is fight to AI */
 		bool is_fight_to_robot(){ return m_status == FIGHT_STATUS;}
+		/** 标识是否网络对战中*/
+		/** check is fight on network */
+		bool is_network_game() { return m_status == NETWORK_STATUS; }
 		/** 检测是不是当前用户走棋 -- check if the human user going to move */
 		bool is_human_player(){ return m_engine.red_player()-m_human_black ; }
+		bool is_rev_game() { return is_rev_board;}
 		/** 走时函数*/
 		/** check go time*/
 		bool go_time();
@@ -167,6 +183,8 @@ class Board : public Gtk::DrawingArea
 		void set_level_config(int _depth,int _idle,int _style,int _knowledge,int _pruning,int _randomness,bool _usebook);
 		/** 设置对战时的走棋时间和总局时间*/
 		void set_war_time(int _step_time,int _play_time);
+		/** 设置保存的走棋时间和总局时间*/
+		void set_time(int _step_time,int _play_time);
 		/** 倒计时的声音*/
 		void reckon_time_sound(int time_);
 		
@@ -203,16 +221,23 @@ class Board : public Gtk::DrawingArea
 		/** 步时 */
 		int m_step;
 		/** 棋局状态*/
-		int m_status;
+		BOARD_STATUS m_status;
 		/** 红方的局时*/
 		int red_time;
 		/** 黑方的局时*/
 		int black_time;
+		/** 用以保存和电脑下棋的局时和步时*/
+		int play_time;
+		int step_time;
 		/** 计时，走秒*/
 		int count_time;
 		/** 每步时的极限秒数*/
 		int limit_count_time;
 
+		/** 接受命令的socket*/
+		int fd_recv_skt;
+		/** 发送命令的socket*/
+		int fd_send_skt;
 		sigc::connection timer;
 		/** 对战状态中标识是否用户走棋,true是用户，false是AI*/
 		//bool user_player;
