@@ -26,8 +26,9 @@
 #include "BookView.h"
 #include "ConfWindow.h"
 #include "gmchess.h"
+#include "Sound.h"
 
-#define version "0.20.5"
+#define version "0.29.2"
 
 Glib::ustring ui_info =
 "<ui>"
@@ -60,7 +61,6 @@ Glib::ustring ui_info =
 MainWindow::MainWindow():menubar(NULL)
 			 ,confwindow(NULL)
 {
-	//ui_xml = Gnome::Glade::Xml::create(main_ui,"main_window");
 	ui_xml = Gtk::Builder::create_from_file(main_ui,"main_window");
 	if(!ui_xml)
 		exit(271);
@@ -81,30 +81,11 @@ MainWindow::MainWindow():menubar(NULL)
 	ui_xml->get_widget("button_lose",btn_lose);
 	ui_xml->get_widget("button_draw",btn_draw);
 	ui_xml->get_widget("button_rue",btn_rue);
-#if 0
-	Gtk::VBox* main_window = dynamic_cast<Gtk::VBox*>(ui_xml->get_widget("main_window"));
-
-	Gtk::VBox* box_board = dynamic_cast<Gtk::VBox*>(ui_xml->get_widget("vbox_board"));
-	buttonbox_war = dynamic_cast<Gtk::ButtonBox*>(ui_xml->get_widget("hbuttonbox_war"));
-	text_comment = dynamic_cast<Gtk::TextView*>(ui_xml->get_widget("textview_comment"));
-
-	m_notebook = dynamic_cast<Gtk::Notebook*>(ui_xml->get_widget("notebook"));
-	 btn_start = dynamic_cast<Gtk::Button*>(ui_xml->get_widget("button_start"));
-	 btn_end = dynamic_cast<Gtk::Button*>(ui_xml->get_widget("button_end"));
-	 btn_prev = dynamic_cast<Gtk::Button*>(ui_xml->get_widget("button_prev"));
-	 btn_next = dynamic_cast<Gtk::Button*>(ui_xml->get_widget("button_next"));
-
-
-	 btn_begin = dynamic_cast<Gtk::Button*>(ui_xml->get_widget("button_begin"));
-	 btn_lose  = dynamic_cast<Gtk::Button*>(ui_xml->get_widget("button_lose"));
-	 btn_draw   = dynamic_cast<Gtk::Button*>(ui_xml->get_widget("button_draw"));
-	 btn_rue  = dynamic_cast<Gtk::Button*>(ui_xml->get_widget("button_rue"));
-#endif
 
 	btn_start->signal_clicked().connect(
-			sigc::mem_fun(*this,&MainWindow::on_start_move));
+			sigc::mem_fun(*this,&MainWindow::on_first_move));
 	btn_end->signal_clicked().connect(
-			sigc::mem_fun(*this,&MainWindow::on_end_move));
+			sigc::mem_fun(*this,&MainWindow::on_last_move));
 	btn_prev->signal_clicked().connect(
 			sigc::mem_fun(*this,&MainWindow::on_back_move));
 	btn_next->signal_clicked().connect(
@@ -119,8 +100,6 @@ MainWindow::MainWindow():menubar(NULL)
 	btn_rue->signal_clicked().connect(
 			sigc::mem_fun(*this, &MainWindow::on_rue_game));
 
-	//init_conf();
-
 	board= Gtk::manage(new Board(*this));
 	box_board->pack_start(*board);
 
@@ -132,14 +111,11 @@ MainWindow::MainWindow():menubar(NULL)
 	/** 设置菜单, set the menu*/
 	init_ui_manager();
 	 menubar = ui_manager->get_widget("/MenuBar");
-	//Gtk::VBox* menu_tool_box	= dynamic_cast<Gtk::VBox*>(ui_xml->get_widget("box_menu_toolbar"));
 	Gtk::VBox* menu_tool_box=0;
 	ui_xml->get_widget("box_menu_toolbar",menu_tool_box);
 	menu_tool_box->pack_start(*menubar,true,true);
 
 	/** 设置Treeview区, set the treeview*/
-	//Gtk::ScrolledWindow* scrolwin=dynamic_cast<Gtk::ScrolledWindow*>
-	//	(ui_xml->get_widget("scrolledwindow"));
 	Gtk::ScrolledWindow* scrolwin= 0;
 	ui_xml->get_widget("scrolledwindow",scrolwin);
 	m_refTreeModel = Gtk::ListStore::create(m_columns);
@@ -154,8 +130,6 @@ MainWindow::MainWindow():menubar(NULL)
 
 	set_status();
 
-	//Gtk::ScrolledWindow* scroll_book=dynamic_cast<Gtk::ScrolledWindow*>
-	//	(ui_xml->get_widget("scrolledwin_book"));
 	Gtk::ScrolledWindow* scroll_book=0;
 	ui_xml->get_widget("scrolledwin_book",scroll_book);
 	m_bookview= new BookView(this);
@@ -167,14 +141,8 @@ MainWindow::MainWindow():menubar(NULL)
 	ui_xml->get_widget("P2_war_time",p2_war_time);
 	ui_xml->get_widget("P1_step_time",p1_step_time);
 	ui_xml->get_widget("P2_step_time",p2_step_time);
-#if 0
-	p1_image = dynamic_cast<Gtk::Image*>(ui_xml->get_widget("image_p1"));
-	p2_image = dynamic_cast<Gtk::Image*>(ui_xml->get_widget("image_p2"));
-	p1_war_time = dynamic_cast<Gtk::Label*>(ui_xml->get_widget("P1_war_time"));
-	p2_war_time = dynamic_cast<Gtk::Label*>(ui_xml->get_widget("P2_war_time"));
-	p1_step_time= dynamic_cast<Gtk::Label*>(ui_xml->get_widget("P1_step_time"));
-	p2_step_time= dynamic_cast<Gtk::Label*>(ui_xml->get_widget("P2_step_time"));
-#endif
+	ui_xml->get_widget("P1_name",p1_name);
+	ui_xml->get_widget("P2_name",p2_name);
 
 	init_conf();
 	this->signal_check_resize().connect(
@@ -188,7 +156,6 @@ MainWindow::MainWindow():menubar(NULL)
 	p1_image->hide();
 	p2_image->hide();
 
-	//set_default_size(851,727);
 	if(atoi(GMConf["desktop_size"].c_str()) == 1)
 		board->set_board_size(BIG_BOARD);
 	else
@@ -198,8 +165,8 @@ MainWindow::MainWindow():menubar(NULL)
 	board->set_level_config(_depth,0,0,0,0,0,atoi(GMConf["usebook"].c_str()));
 	int _step_time= atoi(GMConf["step_time"].c_str());
 	int _play_time= atoi(GMConf["play_time"].c_str());
-	if(_step_time>0&&_step_time<300 && _play_time>0)
-		board->set_war_time(_step_time,_play_time);
+	if(_step_time>0&&_step_time<600 && _play_time>0)
+		board->set_time(_step_time,_play_time);
 }
 
 MainWindow::~MainWindow()
@@ -223,6 +190,7 @@ void MainWindow::change_play(bool player)
 		p2_image->hide();
 		p1_image->show();
 	}
+	
 }
 
 void MainWindow::save_conf()
@@ -247,10 +215,12 @@ void MainWindow::init_conf()
 {
 	char buf[512];
 	char book_dir[512];
+	char file_dir[512];
 	std::string homedir=Glib::get_user_config_dir();
 	snprintf(book_dir, 512,"%s/gmchess/book",homedir.c_str());
 	m_bookview->load_book_dir(book_dir);
 	snprintf(buf,512,"%s/gmchess/config",homedir.c_str());
+
 	std::ifstream file(buf);
 	if(!file){
 		char homepath[512];
@@ -260,6 +230,9 @@ void MainWindow::init_conf()
 		GMConf["desktop_size"] = "1"; //0--small,1--big
 		GMConf["engine_depth"] ="5";
 		save_conf();
+
+		snprintf(file_dir,512,"%s/gmchess/files",homedir.c_str());
+		mkdir(file_dir,S_IRUSR|S_IWUSR|S_IXUSR);
 		return;
 	}
 	
@@ -370,14 +343,14 @@ void MainWindow::on_back_move()
 	board->back_move();
 	show_treeview_step();
 }
-void MainWindow::on_start_move()
+void MainWindow::on_first_move()
 {
-	board->start_move();
+	board->first_move();
 	m_treeview.scroll_to_point(1,1);
 }
-void MainWindow::on_end_move()
+void MainWindow::on_last_move()
 {
-	board->end_move();
+	board->last_move();
 	show_treeview_step();
 }
 
@@ -451,7 +424,6 @@ void MainWindow::on_menu_save_file()
 		return;
 
 
-	//std::ofstream  file("/tmp/chess.pgn");
 	std::ofstream  file;
 	file.open(filename.c_str());
 	if(!file){
@@ -477,6 +449,55 @@ void MainWindow::on_menu_save_file()
 
 }
 
+void MainWindow::auto_save_chess_file()
+{
+	std::string homedir=Glib::get_user_config_dir();
+	homedir += "/gmchess/files/";
+
+	char time1[200];
+	char time2[200];
+	time_t t;
+	struct tm *tmp;
+
+	t = time(NULL);
+	tmp = localtime(&t);
+	strftime(time1, sizeof(time1), "%Y-%m-%d-%H-%M-", tmp);
+	strftime(time2, sizeof(time2), "%Y.%m.%d", tmp);
+	std::string name = p1_name->get_text()+"-"+p2_name->get_text()+".pgn";
+	std::string filename = homedir + std::string(time1) + name;
+
+	std::ofstream  file;
+	file.open(filename.c_str());
+	if(!file){
+		DLOG("open %s file error\n",filename.c_str());
+		return ;
+	}
+	
+	file<<"[Game \"Chese chess Play by GMChess\"]"<<std::endl;
+	file<<"[Date \""<<std::string(time2)<<"\"]"<<std::endl;
+	file<<"[Red \""<<p1_name->get_text()<<"\"]"<<std::endl;
+	file<<"[Black \""<<p2_name->get_text()<<"\"]"<<std::endl;
+
+	Gtk::TreeModel::Children children =  m_refTreeModel->children();
+	Gtk::TreeModel::iterator iter ;
+	for(iter = children.begin();iter!= children.end();iter++){
+
+		file<<(*iter)[m_columns.step_bout] <<". "<<(*iter)[m_columns.step_line] ;
+		iter++;
+		if(iter!=children.end())
+			file<<"  "<<(*iter)[m_columns.step_line]<<std::endl;
+		else{
+			file<<std::endl;
+			break;
+		}
+	}
+
+	file.close();
+
+
+}
+
+
 void MainWindow:: on_menu_open_file()
 {
 	Gtk::FileChooserDialog dlg(*this,
@@ -570,7 +591,6 @@ void MainWindow::open_file(const std::string& filename)
 		size_t pos = filename.find(".pgn");
 		if(pos == std::string::npos){
 			char cmd[1024];
-			//remove("/tmp/gmchess.pgn");
 			sprintf(cmd,"convert_pgn \"%s\" ",filename.c_str());
 			if(system(cmd)<0){
 				DLOG("convert pgn file error\n");
@@ -597,6 +617,15 @@ void MainWindow::open_file(const std::string& filename)
 
 }
 
+void MainWindow::info_window(const std::string& info)
+{
+			Gtk::MessageDialog dialog("Info",false,
+					Gtk::MESSAGE_INFO);
+			dialog.set_secondary_text(info);
+			dialog.run();
+}
+
+
 void MainWindow::on_menu_war_to_ai()
 {
 	on_begin_game();
@@ -616,8 +645,41 @@ void MainWindow::on_menu_rev_play()
 
 void MainWindow::on_menu_file_quit()
 {
+	if(board->get_status() == NETWORK_STATUS){
+		board->send_to_socket("close");
+	}
 	Gtk::Main::quit();
 
+}
+bool MainWindow::on_delete_event(GdkEventAny* event)
+{
+	if(board->get_status() == NETWORK_STATUS){
+		Gtk::MessageDialog dialog(*this, _("Warning"), false,
+                                  Gtk::MESSAGE_QUESTION,
+                                  Gtk::BUTTONS_OK_CANCEL);
+		Glib::ustring msg =_("Are you  sure to close the network game?");
+		dialog.set_secondary_text(msg);
+		int result = dialog.run();
+		switch (result) {
+			case (Gtk::RESPONSE_OK): {
+				board->send_to_socket("close");
+                	        break;
+                	}
+
+			case (Gtk::RESPONSE_CANCEL): {
+			     return true;
+                	        break;
+                	}
+
+			default: {
+			     return true;
+                	        break;
+                	}
+		}
+	}
+
+	return Gtk::Window::on_delete_event(event);
+
 }
 
 void MainWindow::on_menu_view_preferences()
@@ -749,14 +811,9 @@ bool MainWindow::on_treeview_click(GdkEventButton* ev)
 
 void MainWindow::set_information()
 {
-	Gtk::Label* p1_name= 0;ui_xml->get_widget("P1_name",p1_name);
-	Gtk::Label* p2_name= 0;ui_xml->get_widget("P2_name",p2_name);
+	//Gtk::Label* p1_name= 0;ui_xml->get_widget("P1_name",p1_name);
+	//Gtk::Label* p2_name= 0;ui_xml->get_widget("P2_name",p2_name);
 	Gtk::Label* info   = 0;ui_xml->get_widget("info_label",info);
-#if 0
-	Gtk::Label* p1_name= dynamic_cast<Gtk::Label*>(ui_xml->get_widget("P1_name"));
-	Gtk::Label* p2_name= dynamic_cast<Gtk::Label*>(ui_xml->get_widget("P2_name"));
-	Gtk::Label* info   = dynamic_cast<Gtk::Label*>(ui_xml->get_widget("info_label"));
-#endif
 	
 	const Board_info& board_info = board->get_board_info();
 	p1_name->set_label(board_info.black);
@@ -778,21 +835,7 @@ void MainWindow::set_status()
 {
 	int f_status = board->get_status();
 	bool f_use=1;
-#if 0
-	if( READ_STATUS != f_status){
-		f_use = 0;
-		//buttonbox_war->hide();
-	}
-	btn_next->set_sensitive(f_use);
-	btn_prev->set_sensitive(f_use);
-	btn_start->set_sensitive(f_use);
-	btn_end->set_sensitive(f_use);
 
-	btn_begin->set_sensitive(1-f_use);
-	btn_lose->set_sensitive(1-f_use);
-	btn_draw->set_sensitive(1-f_use);
-	btn_rue->set_sensitive(1-f_use);
-#endif
 	switch(f_status){
 		case READ_STATUS:
 			btn_next->set_sensitive(f_use);
@@ -806,6 +849,7 @@ void MainWindow::set_status()
 			btn_rue->set_sensitive(1-f_use);
 			break;
 		case FIGHT_STATUS:
+		case NETWORK_STATUS:
 			btn_next->set_sensitive(1-f_use);
 			btn_prev->set_sensitive(1-f_use);
 			btn_start->set_sensitive(1-f_use);
@@ -827,12 +871,28 @@ void MainWindow::set_status()
 			btn_draw->set_sensitive(1-f_use);
 			btn_rue->set_sensitive(1-f_use);
 			break;
-		case NETWORK_STATUS:
+		default:
 			break;
 	};
 
 }
 
+void MainWindow::on_network_game(const std::string me_name,const std::string& enemy_name,bool role_red_)
+{
+		p1_name->set_text(enemy_name);
+		p2_name->set_text(me_name);
+	if(!role_red_){
+		board->rev_game();
+	}
+	
+	m_refTreeModel->clear();
+	board->start_network();
+	set_status();
+	btn_begin->set_sensitive(false);
+		
+	
+}
+
 void MainWindow::on_begin_game()
 {
 	/** 已经在对战中，则询问是否开始新游戏*/
@@ -862,6 +922,14 @@ void MainWindow::on_begin_game()
 		}
 		return;
 
+	}
+	else if(board->is_network_game()){
+		Gtk::MessageDialog dialog_info(*this, _("Information"), false);
+		Glib::ustring msg =_("You are play with network game,Please over it first!");
+		dialog_info.set_secondary_text(msg);
+		int result = dialog_info.run();
+		return ;
+
 	}
 	m_refTreeModel->clear();
 	board->start_robot();
@@ -873,21 +941,23 @@ void MainWindow::on_lose_game()
 {
 
     if(board->is_fight_to_robot()){
-        Gtk::MessageDialog dialog(*this, _("end game"), false,
+        Gtk::MessageDialog dialog(*this, _("be lose"), false,
                                   Gtk::MESSAGE_QUESTION,
                                   Gtk::BUTTONS_OK_CANCEL);
-        Glib::ustring msg =_("Will you end this game?");
+        Glib::ustring msg =_("Will you resign in this game?");
         dialog.set_secondary_text(msg);
         int result =dialog.run();
         switch (result) {
             case (Gtk::RESPONSE_OK): {
-                m_refTreeModel->clear();
+                //m_refTreeModel->clear();
                 board->free_game();
                 set_status();
                             break;
                     }
 
             case (Gtk::RESPONSE_CANCEL): {
+                //board->free_game(false);
+                //set_status();
                             break;
                     }
 
@@ -897,12 +967,55 @@ void MainWindow::on_lose_game()
         }
         return;
     }
+    else if(board->is_network_game()){
+
+        Gtk::MessageDialog dialog(*this, _("be lose"), false,
+                                  Gtk::MESSAGE_QUESTION,
+                                  Gtk::BUTTONS_OK_CANCEL);
+        Glib::ustring msg =_("Will you resign in this game?");
+        dialog.set_secondary_text(msg);
+        int result =dialog.run();
+        switch (result) {
+            case (Gtk::RESPONSE_OK): {
+		    board->send_to_socket("resign");
+		Gtk::MessageDialog dialog_info(*this, _("Information"), false);
+		Glib::ustring msg =_("You lose the game!");
+		dialog_info.set_secondary_text(msg);
+		int result = dialog_info.run();
+                board->free_game();
+                set_status();
+                            break;
+                    }
+
+            case (Gtk::RESPONSE_CANCEL): {
+                //board->free_game(false);
+                //set_status();
+                            break;
+                    }
+
+            default: {
+                            break;
+                    }
+        }
+    }
 }
 
 /** draw 是打平局面的意思*/
 void MainWindow::on_draw_game()
 {
-	board->draw_move();
+    if(board->is_fight_to_robot()){
+		board->draw_move();
+    }
+	else if(board->is_network_game()){
+		if(board->is_human_player()){
+			board->send_to_socket("draw");
+		}else{
+			Gtk::MessageDialog dialog(*this, _("Warning"), false);
+			Glib::ustring msg =_("Only ask rue game when you are going,so wait a minute!");
+			dialog.set_secondary_text(msg);
+			int result = dialog.run();
+		}
+	}
 
 }
 
@@ -918,7 +1031,19 @@ void MainWindow::on_rue_game()
 			board->rue_move();
 		}else{
 			Gtk::MessageDialog dialog(*this, _("Warning"), false);
-			Glib::ustring msg =_("Only rue game when you are go,so wait a minute!");
+			Glib::ustring msg =_("Only rue game when you are going,so wait a minute!");
+			dialog.set_secondary_text(msg);
+			int result = dialog.run();
+		}
+
+	}
+	else if(board->is_network_game()){
+		if(board->is_human_player()){
+			/** 发送了悔棋指令，此时界面不应该能操作，直到对方返回应答，yes/no */
+			board->send_to_socket("rue");
+		}else{
+			Gtk::MessageDialog dialog(*this, _("Warning"), false);
+			Glib::ustring msg =_("Only rue game when you are going,so wait a minute!");
 			dialog.set_secondary_text(msg);
 			int result = dialog.run();
 		}
@@ -934,59 +1059,87 @@ bool MainWindow::on_end_game(OVERSTATUS _over)
 		case ROBOT_WIN:
 			//robot win
 			msg=_("You are Lose! \n Are you want to start a new game?");
+			CSound::play(SND_LOSS);
 			break;
 		case ROBOT_LOSE:
 			//robot lose
 			msg=_("Congratuations ! YOU WIN!\n start a new game click OK");
+			CSound::play(SND_WIN);
 			break;
 		case ROBOT_DRAW:
 			//robot want to draw
-			msg=_("Computer want to draw with you, are you agree?");
+			msg=_("Draw Game!");
 			break;
 		case ROBOT_OVER_TIME:
 			// robot overload time,lose
 			msg = _("The Enemy overload the time, You Win!");
+			CSound::play(SND_WIN);
 			break;
 		case HUMAN_OVER_TIME:
 			msg = _("You overload the time, You Lose!");
+			CSound::play(SND_LOSS);
 		default:
 			break;
 
 	}
-	Gtk::MessageDialog dialog(*this, _("Game End"), false,
-                                  Gtk::MESSAGE_QUESTION,
-                                  Gtk::BUTTONS_OK_CANCEL);
-		//Glib::ustring msg =_("Mate!\n Are you want to start a new game?");
-		dialog.set_secondary_text(msg);
-		int result = dialog.run();
-		switch (result) {
-			case (Gtk::RESPONSE_OK): {
-				m_refTreeModel->clear();
-				board->new_game();
-				return true;
-                	}
-
-			case (Gtk::RESPONSE_CANCEL): {
-				return false;
-                	}
-
-			default: {
-				return false;
-                	        break;
-                	}
-		}
+		Gtk::MessageDialog dialog_info(*this, _("Game End"), false);
+		dialog_info.set_secondary_text(msg);
+		int result = dialog_info.run();
 
+		if(board->is_network_game())
+			auto_save_chess_file();
+		board->free_game(false);
+		set_status();
 }
 
 void MainWindow::set_red_war_time(const Glib::ustring& f_time,const Glib::ustring& c_time)
 {
-	p2_war_time->set_text(f_time);
-	p2_step_time->set_text(c_time);
+
+		p2_war_time->set_text(f_time);
+		p2_step_time->set_text(c_time);
 
 
 }
 void MainWindow::set_black_war_time(const Glib::ustring& f_time,const Glib::ustring& c_time)
 {
-	p1_war_time->set_text(f_time);
-	p1_step_time->set_text(c_time);
+
+		p1_war_time->set_text(f_time);
+		p1_step_time->set_text(c_time);
+}
+void MainWindow::watch_socket(int fd)
+{
+	board->watch_socket(fd);
+	
+}
+void MainWindow::start_with(const std::string& param)
+{
+		size_t pos;
+		if((pos = param.find("network-game-red,"))!= std::string::npos){
+			//start network game with red player
+			std::string enemy_name,my_name;
+			size_t pos_s,pos_e,pos_m;
+			pos_s= param.find("enemy_name:");
+			pos_e= param.find(",my_name:");
+			pos_m= param.find_first_of("@");
+			enemy_name = param.substr(pos_s+11,pos_m-pos_s-11);
+			pos_m = param.find_last_of("@");
+			my_name = param.substr(pos_e+9,pos_m-pos_e-9);
+			
+			on_network_game(enemy_name,my_name,true);
+		}else if((pos = param.find("network-game-black,")) != std::string::npos){
+			//start network game with black player
+			std::string enemy_name,my_name;
+			size_t pos_s,pos_e,pos_m;
+			pos_s= param.find("enemy_name:");
+			pos_e= param.find(",my_name:");
+			pos_m= param.find_first_of("@");
+			enemy_name = param.substr(pos_s+11,pos_m-pos_s-11);
+			pos_m = param.find_last_of("@");
+			my_name = param.substr(pos_e+9,pos_m-pos_e-9);
+			on_network_game(my_name,enemy_name,false);
+		}
+		else{
+			if(!param.empty())
+				open_file(param);
+		}
 }
